package com.altinity.opsview.odw;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.io.File;
import java.util.Calendar;
import java.util.Date;

import com.excira.utils.DateUtils;
import com.excira.utils.StringUtils;

public class CommandLineParams {
	
	public static final String N_REGX = "[0]|-\\d*";
	public static final String YYYY_MM_REGX = "\\d{4}-(0[1-9]|1[012])";
	public static final String YYYY_MM_DD_REGX = "\\d{4}-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[01])";
	
	private String reportId ="";
	private String fileName = "";
	private String userName = "";
	private String password = "";
	private String strStartDate = "";
	private String outputFilePath;
	
	private int startOfWeek;
	
	/**
	 * 
	 */
	public CommandLineParams(String [] args) {
				
		int k = 0;
		while ( args.length > k ) {
			if ( args[k].startsWith("-N") )
				reportId = args[k].substring(2);
			else if ( args[k].startsWith("-F") )
				fileName = args[k].substring(2);			
			else if ( args[k].startsWith("-S") )		
				strStartDate = args[k].substring(2);
			else if ( args[k].startsWith("-P") )
				password = args[k].substring(2);
			else if ( args[k].startsWith("-U") )
				userName = args[k].substring(2);
			else if ( args[k].startsWith("-f") )
				outputFilePath = args[k].substring(2);
			k++;	
		}
	}
	
	/**
	 * 
	 */
	public String getStrStartDate() {
		return strStartDate;
	}
	
	/**
	 * 
	 */
	public String getFileName() {
		return fileName;
	}
	
	/**
	 *
	 */
	public Date getRunDate() {
		return DateUtils.getNow();
	}
	

	/**
	 * 
	 */
	public String getReportId() {
		return reportId;
	}	
	
	/**
	 * 
	 */
	public boolean isValid() {

		// Check required parameters have been specified correctly
		if ( fileName.equals("") || reportId.equals("") )
			return false;
		
		// If either a user name or password has been specified,
		// confirm both have been specified
		if ( StringUtils.notNullOrEmpty(userName) && StringUtils.isNullOrEmptyString(password)
				|| StringUtils.notNullOrEmpty(password) && StringUtils.isNullOrEmptyString(userName) )
			return false;
		
		// Confirm file exists
		File file = new File(fileName);
		if ( !file.exists() )
			return false;
		
		// Check the start date
		if ( !isStartDateValid() )
			return false;
		
		// Confirm no spaces have been specified
		// at the beginning of the output directory
		if ( outputFilePath != null && 
				(outputFilePath.startsWith(" ") || outputFilePath.equals("")) )
			return false;
		
		return true;
	}
	
	/**
	 * 
	 */
	private boolean isStartDateValid() {
		
		// First check if a start date has been specified
		// If so, confirm it was specified in correct format
		if ( StringUtils.notNullOrEmpty(strStartDate) ) {
			 
			// Check start date is in yyyy-MM, yyyy-MM-dd or -n format
			if  ( !DateUtils.isValidDate(strStartDate, CommandLineParams.YYYY_MM_REGX, "yyyy-MM")
					&& !DateUtils.isValidDate(strStartDate, CommandLineParams.YYYY_MM_DD_REGX, "yyyy-MM-dd")
					&& !StringUtils.matchesPattern(strStartDate, CommandLineParams.N_REGX) )
				return false;
			
			// If yyyy-MM format, confirm specified month is this month or earlier
			if ( StringUtils.matchesPattern(strStartDate, CommandLineParams.YYYY_MM_REGX) ) {
				
				// Get the specified calendar
				Calendar specifiedCal = Calendar.getInstance();
				specifiedCal.setTime(DateUtils.parse(strStartDate, "yyyy-MM"));
				DateUtils.clearHourMinuteSecond(specifiedCal);
				
				// Get the current calendar
				Calendar cal = Calendar.getInstance();
				DateUtils.clearHourMinuteSecond(cal);
				
				if ( specifiedCal.compareTo(cal) > 0 )
					return false;				
			}
			
			// If yyyy-MM-dd format, confirm specified day and month is earlier than today
			if ( StringUtils.matchesPattern(strStartDate, CommandLineParams.YYYY_MM_DD_REGX) ) {
				
				// Get the specified calendar
				Calendar specifiedCal = Calendar.getInstance();
				specifiedCal.setTime(DateUtils.parse(strStartDate, "yyyy-MM-dd"));
				DateUtils.clearHourMinuteSecond(specifiedCal);
				
				// Get the current calendar
				Calendar cal = Calendar.getInstance();
				DateUtils.clearHourMinuteSecond(cal);
				
				if ( specifiedCal.compareTo(cal) > 0 )
					return false;
			}
		}
		
		return true;
	}
	
	/**
	 *
	 */
	public void usage() {
		
		System.out.println("ODWReportLauncher usage:");
		System.out.println("\tjava -jar ODWReportLauncher.jar (-UuserName -Ppassword) -NreportId -FconfigFile (-foutputFilePath) (-SstartDate)");
		System.out.println("\tStart date format options:");
		System.out.println("\t\tyyyy-MM (where MM is <= current month). Use this format for Availability Reports.");
		System.out.println("\t\tyyyy-MM-dd (where MM-dd is <= today). Use this format for Weekly Service Reports.");
		System.out.println("\t\t-n (where n is an integer). Use this format for either Availability or Weekly Service Reports.");
		System.out.println("\tIf specifying authentication credentials, both user name and password must be specified");
		System.out.println("\tIf specifying output file path, confirm file path already exists");		
		System.out.println("\tConfirm file path to config file is correct");
	}
	
	/**
	 *
	 */
	public String getPassword() {
		return password;
	}
	
	/**
	 *
	 */
	public void setPassword(String password) {
		this.password = password;
	}
	
	/**
	 *
	 */
	public String getUserName() {
		return userName;
	}
	
	/**
	 *
	 */
	public void setUserName(String userName) {
		this.userName = userName;
	}
	
	/**
	 *
	 */
	public int getStartOfWeek() {
		return startOfWeek;
	}
	
	/**
	 *
	 */
	public void setStartOfWeek(int startOfWeek) {
		this.startOfWeek = startOfWeek;
	}
	
	/**
	 *
	 */
	public String getOutputFilePath() {
		return outputFilePath;
	}
	
	/**
	 *
	 */
	public void setOutputFilePath(String outputDir) {
		this.outputFilePath = outputDir;
	}
}