package com.altinity.opsview.odw.bc;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.altinity.opsview.odw.exception.NoDescriptionFoundForHostException;
import com.altinity.opsview.odw.exception.NoPriorStatusFoundForHost;
import com.altinity.opsview.odw.exception.UnableToDetermineHourSummaryForHostException;
import com.altinity.opsview.odw.model.ChartData;
import com.altinity.opsview.odw.model.Host;
import com.altinity.opsview.odw.model.HourSummary;
import com.altinity.opsview.odw.model.PerformanceData;
import com.altinity.opsview.odw.model.ServiceCheckConfig;
import com.altinity.opsview.odw.model.StateHistory;
import com.altinity.opsview.odw.model.StateHistoryBreakdown;
import com.altinity.opsview.odw.model.StateHistoryDisplay;
import com.altinity.opsview.odw.model.SummaryData;
import com.excira.utils.DateUtils;

public class ODWReportBC extends BaseReportBC{

	public static final int OK = 4;
	public static final int WARNING = 3;
	public static final int UNKNOWN = 2;
	public static final int CRITICAL = 1;
	public static final int NO_DATA = 0;

	public static final String STATUS_OK = "OK";
	public static final String STATUS_WARNING = "WARNING";
	public static final String STATUS_UNKNOWN = "UNKNOWN";
	public static final String STATUS_CRITICAL = "CRITICAL";
	public static final String STATUS_NO_DATA = "INDETERMINATE";

	public static final String OSXUNIX = "OSX/Unix";
	public static final String WINDOWS = "Windows";

	public static final String CPU = "CPU";
	public static final String MEMORY = "Memory";
	public static final String STORAGE = "Storage";
	
	public static final int [] RGB_OK = new int [] {0, 128, 0};
	public static final int [] RGB_WARNING = new int [] {255, 210, 0};
	public static final int [] RGB_UNKNOWN = new int [] {255, 153, 0};
	public static final int [] RGB_CRITICAL = new int [] {255, 0, 0};
	public static final int [] RGB_NO_DATA = new int [] {200, 200, 200};
	
	public static final String HEX_WARNING = "ffd200";

	public static final String CHART_DATE_FORMAT = "EEE MMM d HH:mm:ss yyyy";
	public static final String ERROR_LABEL = "####";
	
	private int binCount = 900;

	/**
	 * 
	 */
	public List<SummaryData> getSummaryData(List<Host> hosts) {

		// Get the performance data for the summary table
		// We will need to make a map of host name to
		// SummaryData objects - we will get the SummaryData objects
		// out as a list later
		Map<String, SummaryData> summaryDataByHost = new LinkedHashMap<String, SummaryData>();

		// For each host, get performance value for
		// each performance label
		for (Host host : hosts) {

			String hostName = host.getName();

			// For each service check config, get the service check name and
			// query
			// the db
			List<ServiceCheckConfig> serviceCheckConfigs = host.getServiceCheckConfigs();
			for (ServiceCheckConfig serviceCheckConfig : serviceCheckConfigs) {
				
				String serviceCheckName = serviceCheckConfig.getServiceCheck().getName();

				List<PerformanceData> perfData = 
					dao.queryPerformanceData(hostName, serviceCheckName, 
							serviceCheckConfig.getPerformanceLabel().getName(), 
							reportCriteria.getReportStartDate(), reportCriteria.getReportEndDate());
				
				// Create SummaryData object if it does not already exist for
				// this host
				SummaryData summaryData = summaryDataByHost.get(hostName);
				if (summaryData == null) {
					summaryData = new SummaryData(hostName);
					
					// Query for host description
					try {
						summaryData.setHostDescription(dao.queryHostDescription(hostName));
					}
					catch(NoDescriptionFoundForHostException e) {
						log.log("Unable to find description for host '" + hostName + "'");
						summaryData.setHostDescription(ODWReportBC.ERROR_LABEL);
					}
					summaryDataByHost.put(hostName, summaryData);
				}
				
				String serviceCheckType = serviceCheckConfig.getType();
				summaryData.setPerformanceLabel(serviceCheckType, serviceCheckConfig.getPerformanceLabel());
				
				// Log error if there is no performance data for this host name
				// and service check combination 
				if ( perfData.isEmpty() ) {
					log.log("No performance data found for service check '" + serviceCheckName + 
							"' on host '" + hostName + "'");
				}
				// Otherwise add calculate average and add to summary data for 
				// this host
				else 
					summaryData.setServiceCheckValue(serviceCheckType, calculateAverage(perfData));
			}
		}
		return new ArrayList<SummaryData>(summaryDataByHost.values());
	}

	/**
	 * 
	 */
	private Double calculateAverage(List<PerformanceData> performanceData) {

		double dividend = 0.0d;
		double divisor = 0.0d;

		// Get first date - need this to check initial interval
		long previous = performanceData.get(0).getDate();

		// Iterate over performance data list and add value
		// to total so far
		for (int i = 0; i < performanceData.size(); i++) {

			PerformanceData perfData = performanceData.get(i);

			double value = perfData.getValue();
			long current = perfData.getDate();

			// Ignore any intervals greater than five minutes
			if (ignoreInterval(previous, current)) {

				// Set previous interval start time to be the next
				// interval start time and continue
				if (i + 1 < performanceData.size())
					previous = performanceData.get(i + 1).getDate();
				continue;
			}

			// Otherwise weight value and add to total
			dividend = dividend + (value * current);
			divisor = divisor + current;

			// Set current date to be previous date
			previous = current;
		}

		// Round to two decimal places
		return new Double(dividend / divisor);
	}

	/**
	 * Ignore any intervals longer than 5 minutes
	 */
	private boolean ignoreInterval(long previous, long current) {

		// Determine the interval in minutes
		double diffMinutes = (current - previous) / (60.0 * 1000.0);
		return diffMinutes >= 5.0;
	}

	/**
	 * 
	 */
	public List<ChartData> getChartData(List<Host> hosts) {

		// Get the state history
		List<ChartData> charts = new ArrayList<ChartData>();

		// For each host, get the state history for each service check
		for (Host host : hosts) {

			String hostName = host.getName();

			// For each service check config, get the service check name and
			// query
			// the db
			List<ServiceCheckConfig> serviceCheckConfigs = host.getServiceCheckConfigs();
			for (ServiceCheckConfig serviceCheckConfig : serviceCheckConfigs) {

				String serviceCheckName = serviceCheckConfig.getServiceCheck().getName();

				List <StateHistory> stateHistory = 
					dao.queryStateHistory(hostName, serviceCheckName, 
							reportCriteria.getReportStartDate(), reportCriteria.getReportEndDate());
				
				// If there is no transitions, check the hourly summary for the first 
				String priorStatus = ODWReportBC.STATUS_NO_DATA;
				
				// Handle case where there are no transitions
				if ( stateHistory.isEmpty() ) {
					
					try {
						HourSummary summary = 
							dao.queryHourSummary(hostName, serviceCheckName, reportCriteria.getReportStartDate());
						priorStatus = summary.getStatus();
					}
					catch(UnableToDetermineHourSummaryForHostException e) {
						log.log("Unable to determine previous status using hour summary for " +
								"service check '" + serviceCheckName + "' on host '" + hostName + "'");
					}
				}
				// Otherwise get the time stamp of the first transition and use it get the prior status
				else {
					
					try {
					
						priorStatus = dao.queryPriorStatus(hostName, serviceCheckName, new Date(stateHistory.get(0).getDate()));
					}
					catch(NoPriorStatusFoundForHost e) {
						log.log("No prior status found for " +
								"service check '" + serviceCheckName + "' on host '" + hostName + "'");
					}
				}

				// Create ChartData object if it does not already exist for this
				// host
				ChartData chartData = new ChartData(hostName, serviceCheckName);
				chartData.setStateHistory(expandStateHistory(stateHistory, priorStatus));
				chartData.setBreakdownsByType(calculateStateBreakdowns(stateHistory));

				charts.add(chartData);
			}
		}

		return charts;
	}
	
	/**
	 * 
	 */
	private Map<Integer, StateHistoryBreakdown> calculateStateBreakdowns(List<StateHistory> stateHistory) {

		// Create map containing four state history breakdowns
		Map<Integer, StateHistoryBreakdown> breakdowns = new HashMap<Integer, StateHistoryBreakdown>();
		breakdowns.put(ODWReportBC.OK, new StateHistoryBreakdown());
		breakdowns.put(ODWReportBC.WARNING, new StateHistoryBreakdown());
		breakdowns.put(ODWReportBC.UNKNOWN, new StateHistoryBreakdown());
		breakdowns.put(ODWReportBC.CRITICAL, new StateHistoryBreakdown());
		breakdowns.put(ODWReportBC.NO_DATA, new StateHistoryBreakdown());
		
		Date reportStartDate = reportCriteria.getReportStartDate();
		Date reportEndDate = reportCriteria.getReportEndDate();

		long startTime = reportStartDate.getTime();
		long timeOfLastTransition = startTime;
		int currentStatus = ODWReportBC.OK; // TODO

		// First check if there are no state changes
		if (stateHistory.isEmpty())
			breakdowns.get(currentStatus).addDuration(reportEndDate.getTime() - startTime);
		else {

			// Calculate percentage duration and percentage in each state
			for (StateHistory s : stateHistory) {

				// If this status is the same as the current, then the
				// transition is not "real"
				if (s.getStatusInt() == currentStatus)
					continue;

				// Get the duration of the last transition
				// and add to duration so far for this state
				long duration = s.getDate() - timeOfLastTransition;
				breakdowns.get(currentStatus).addDuration(duration);

				// Get the time of this transition
				timeOfLastTransition = s.getDate();

				// Update status
				currentStatus = s.getStatusInt();
			}

			// Now add last transition to the end time to the current state
			long finalDuration = reportEndDate.getTime() - timeOfLastTransition;
			breakdowns.get(currentStatus).addDuration(finalDuration);
		}

		// Now calculate percentages
		long totalTime = reportEndDate.getTime() - reportStartDate.getTime();
		for (Integer status : breakdowns.keySet()) {

			StateHistoryBreakdown breakdown = breakdowns.get(status);
			breakdown.setPercent(new Double(breakdown.getDuration()) / new Double(totalTime) * 100);
		}

		return breakdowns;
	}

	/**
	 * 
	 */
	private List<StateHistoryDisplay> expandStateHistory(List<StateHistory> stateHistory, String priorStatus) {
		
		Date reportStartDate = reportCriteria.getReportStartDate();
		Date reportEndDate = reportCriteria.getReportEndDate();

		// First determine the interval of each bin
		// endDate - startDate / binCount
		long binDuration = (reportEndDate.getTime() - reportStartDate.getTime()) / binCount;

		// Create a new list of state history display objects that will act as
		// bins to display transitions
		List<StateHistoryDisplay> bins = new ArrayList<StateHistoryDisplay>(binCount);

		long binStartDate = reportStartDate.getTime();
		long binEndDate = binStartDate + binDuration;

		// Create initial transition at start time and
		// add to beginning of state history
		StateHistory firstTransition = new StateHistory();
		firstTransition.setDate(binStartDate);
		boolean first = true;
		firstTransition.setStatus(priorStatus);
		stateHistory.add(0, firstTransition);

		int lastStatus = firstTransition.getStatusInt();

		// Iterate over transitions list while creating bins
		// If transition falls in to bin, BIN IT!
		StateHistoryDisplay bin = null;
		for (StateHistory transitionToBin : stateHistory) {

			long transitionDate = transitionToBin.getDate();

			// Create bins until transition is binned
			while (bins.size() < binCount) {

				// Create bin and set lastStatus as its default status
				// This may be overridden if another transition falls
				// into this bin
				if (bin == null || (transitionToBin.getDate() > binEndDate)) {
					bin = new StateHistoryDisplay();
					bin.setStatus(lastStatus);
					if (!first) {

						binStartDate = binEndDate;
						binEndDate = binStartDate + binDuration;

					}
					first = false;
					bins.add(bin);
				}

				// Transition falls into this bin - update the bin
				// with transition values
				if (transitionDate >= binStartDate && transitionDate < binEndDate) {

					int transitionStatus = transitionToBin.getStatusInt();
					
					// Update the bin if the transition status is worse than
					// the current bin status OR update the status 
					// if the current bin status is indeterminate
					if (transitionStatus < bin.getStatus() || 
							bin.getStatus() == ODWReportBC.NO_DATA ) {
						bin.setStatus(transitionStatus);
						
					}
					
					if (bin.isVirgin())
						bin.setDate(DateUtils.format(new Date(transitionDate), CHART_DATE_FORMAT));

					// House keeping
					lastStatus = transitionToBin.getStatusInt();

					break;
				}
			}
		}

		// all transitins are binned keep same stauts till end of chart or set
		// to no data if chart goes
		// beyond available data.
		while (bins.size() < binCount) {
			
			binStartDate = binEndDate;
			binEndDate = binStartDate + binDuration;
			bin = new StateHistoryDisplay();
			bins.add(bin);
			// check if bin is past current time
			if (binStartDate > runDate.getTime()) {
				bin.setStatus(ODWReportBC.NO_DATA);
				if (lastStatus != ODWReportBC.NO_DATA) {
					lastStatus = ODWReportBC.NO_DATA;
					//set date marker for last valid time in chart
					bin.setDate(DateUtils.format(runDate, CHART_DATE_FORMAT));
				}
			}else{
				bin.setStatus(lastStatus);
			}
			

		}

		return bins;
	}
}
