package com.altinity.opsview.odw.elements;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.dom4j.Element;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryLabelPositions;

import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.bc.ODWReportLogger;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.jfreechart.BarChart;
import com.excira.jfreechart.BarData;
import com.excira.jfreechart.DateChartData;
import com.excira.utils.DateUtils;

public class AvailabilityTrendChart extends AbstractReportElement implements ReportConstants {
	
	private List <List<BarData>> barData;

	/**
	 * 
	 */
	public AvailabilityTrendChart(ReportCriteria rc, ODWReportDAO dao, ODWReportConfigDAO configDao, ODWReportLogger log) {
		super(rc, dao, configDao, log);
	}
	
	/**
	 * 
	 */
	@Override
	public Element addElement(Element elements) {
		
		// First create element
		Element availabilityTrendGraph = elements.addElement("element");
		availabilityTrendGraph.addAttribute("name", "availabilityTrendGraph");
		availabilityTrendGraph.addElement("title").setText("Availability Trend, Last 4 Months ");
		
		// If no data has been found, add empty element and report error
		if ( barData.isEmpty() ) {
			
			// Add empty element
			availabilityTrendGraph.addElement("empty").setText("true");
			
			// Report error
			log.getErrors().add("No data found for module 'Availability Trend, Last 4 Months'");
		}
		// Data has been found
		// Generate average availability chart
		else {

			// Now, we want to make sure that all weeks are included in the list,
			// even weeks that have no data
			List unpackedData = padDataAcrossFourMonths();
			
			BarChart barChart = new BarChart();
			barChart.createDomainAxis("Week Beginning", CategoryLabelPositions.UP_90);
			JFreeChart chart = barChart.createChart("", "Availability %", unpackedData, false);
			String fileName = saveImageToTempDir(chart, "availabilityTrend");
			availabilityTrendGraph.addElement("image").setText(fileName);
		}		
		
		return availabilityTrendGraph;
	}
	
	/**
	 * 
	 */
	private List<BarData> padDataAcrossFourMonths() {
		
		// Create a calendar that we can use to determine
		// how many extra weeks we need to pad the list by
		Calendar actualStart = Calendar.getInstance(); 
		actualStart.setTime(rc.getReportStartDate());
		actualStart.add(Calendar.MONTH, -3);
		DateUtils.adjustDateAccordingToWeekConfigToGetFirstDayOfWeek(actualStart, Calendar.MONDAY);
		DateUtils.clearHourMinuteSecond(actualStart);
				
		// Now pad the beginning of the list
		// for any missing weeks
		Date firstInReport = ((DateChartData)barData.get(0).get(0).getColumnKey()).getDate();
		
		int index = 0;
		while ( DateUtils.isBefore(actualStart.getTime(), firstInReport) ) {
			
			DateChartData dateData = new DateChartData(actualStart.getTime());
			dateData.setPattern("dd-MMM-yy");
			
			barData.add(index, createBarDataForDate(0, 0, 0, dateData));
			
			// Increment calendar and index
			index++;
			actualStart.add(Calendar.WEEK_OF_YEAR, 1);
		}
		
		// Now pad out the end of the list
		// for any missing weeks
		Calendar actualEnd = Calendar.getInstance();
		actualEnd.setTime(rc.getReportEndDate());
		DateUtils.adjustDateAccordingToWeekConfigToGetFirstDayOfWeek(actualEnd, Calendar.MONDAY);
		DateUtils.clearHourMinuteSecond(actualEnd);
		Date actualEndDate = actualEnd.getTime();
				
		Calendar lastInReport = Calendar.getInstance();
		lastInReport.setTime(((DateChartData)barData.get(barData.size() -1).get(0).getColumnKey()).getDate());
		
		// Add one week on last in report
		// This is because the last in report should have already been included
		lastInReport.add(Calendar.WEEK_OF_YEAR, 1);
		
		while ( DateUtils.isBefore(lastInReport.getTime(), actualEndDate) 
				|| DateUtils.isDateEqual(lastInReport.getTime(), actualEndDate) ) {
			
			DateChartData dateData = new DateChartData(lastInReport.getTime());
			dateData.setPattern("dd-MMM");
			
			barData.add(createBarDataForDate(0, 0, 0, dateData));
			
			// Increment calendar and index
			lastInReport.add(Calendar.WEEK_OF_YEAR, 1);
		}
		
		// Unpack bar data
		List <BarData> unpacked = new  ArrayList<BarData>();
		for ( List <BarData> list : barData ){
			
			unpacked.addAll(list);
		}
		
		return unpacked;
	}
	
	/**
	 * 
	 */
	private List<BarData> createBarDataForDate(double ok, double sched, double unsched, Comparable xKey) {
		
		List <BarData> data = new ArrayList<BarData>(3);
		
		BarData barData = new BarData(ok, ODWReportDAO.PERCENT_OK_INDEX, xKey);
		data.add(barData);
		
		barData = new BarData(sched, ODWReportDAO.PERCENT_SCHED_INDEX, xKey);
		data.add(barData);
		
		barData = new BarData(unsched, ODWReportDAO.PERCENT_UNSCHED_INDEX, xKey);
		data.add(barData);
		
		return data;
	}
	
	/**
	 * 
	 */
	public void runQuery() throws ReportException {
		
		barData = dao.getAvailabilityTrendData(configDao, rc);
	}
}
